package org.netkernel.layer0.bnf;

import org.netkernel.container.IMessages;
import org.netkernel.layer0.meta.GrammarException;
import org.netkernel.layer0.util.OrderedCheapMultiStringMap;

/** A Grammar Group is a grammar element that can contain sub-parts
 * @author tab
 */
public interface IGrammarGroup extends IGrammarPart
{	
	/** Return the name of the group, null if no name
	 */
	String getName();
	
	/** Return the encoding used by this part if any
	 */
	IPartEncoding getEncoding();
	
	/** Return the parent group
	 */
	IGrammarGroup getParent();
	
	/** Return the number of nested parts
	 */
	int getPartCount();
	
	int getCapturedGroupIndexInParent();
	
	/** Return the nested part with given index
	 */
	IGrammarPart getPart(int aIndex);

	/** Return the regular expression string for this group and it's nested parts
	 */
	String getRegex();
	
	/** Returns true if we must check deeper into this group to confirm a match, my calling matches()
	 */
	boolean needsToMatchDeeper();

	/** Return true if this group matches given the set of captured groups offset by the given amount
	 * @param aCapturedGroups
	 * @param aOffset
	 * @return
	 */
	boolean matches(String[] aCapturedGroups, int aOffset);
	
	/** Return an array of captured groups for this grammar group
	 * @param aIdentifier the identifier to match against
	 */
	String[] matchesForGroups(String aIdentifier);
	
	/** Match just this part
	 * @param aIdentifier
	 * @return
	 */
	boolean match(String aIdentifier);
	
	String constructIdentifier(OrderedCheapMultiStringMap aPartsRemaining, IMessages aMessages) throws GrammarException;
	
	/** Parse captured groups (offset by aOffset) into the result map
	 * @param aCapturedGroups
	 * @param aResults
	 * @param aOffset
	 */
	void parse(String[] aCapturedGroups, OrderedCheapMultiStringMap aResults, int aOffset);
	
	
	/** Add a nested part to this group. May only be called before close()
	 * @param aPart
	 */
	void addPart(IGrammarPart aPart);
	
	/** Prepare group ready for use
	 * @param aMessages
	 * @throws GrammarParseException
	 */
	void close(IMessages aMessages) throws GrammarParseException;
	
	void close2();
}
