/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf;

import org.netkernel.container.ILogger;

/** Interface to formatting strings, log messages and exceptions in a local specific way
 *  Log messages will issued to an appropriate logger
 * @author Tony Butterfield
 */
public interface INKFLocale
{
	public static final int LEVEL_SEVERE=ILogger.SEVERE;
	public static final int LEVEL_WARNING=ILogger.WARNING;
	public static final int LEVEL_INFO=ILogger.INFO;
	public static final int LEVEL_FINE=ILogger.FINE;
	public static final int LEVEL_FINER=ILogger.FINER;
	public static final int LEVEL_DEBUG=ILogger.DEBUG;
	
	/** Create a formatted exception
	 * @param aId The id of the message that will be looked up
	 * @param aMessage Optional message body of the exception that will be looked up
	 * @param aMessageParams Optional parameters that will be substituted into the message
	 * @param aCause An optional underlying cause java exception, may be null
	 * @return the exception
	 */
	NKFException createFormattedException(String aId, String aMessage, String aLocation, Throwable aCause, Object... aMessageParams);

	/** Format a message from the message bundle
	 * @param aMessageKey The id of the message that will be looked up
	 * @param aParams optional parameters to be substituted into the message
	 * @return the formatted message
	 */
	String formatMessage(String aMessageKey, Object... aParams);	
	
	/** Format a literal message
	 * @param aMessage The string of the message with percent substitution points
	 * @param aParams optional parameters to be substituted into the message
	 * @return the formatted message
	 */
	String formatRaw(String aMessage, Object... aParams);
	
	/** Check to see if it is worth preparing to log a message 
	 * @param aLevel the level of the message
	 * @return
	 */
	boolean shouldLog(int aLevel);
	
	/** Check to see if it is worth preparing to log a message 
	 * @param aLog path of resource of custom logger config (e.g. "/etc/system/CustomLogConfig.xml")
	 * @param aLevel the level of the message
	 * @return
	 */
	boolean shouldLog(String aLog, int aLevel);

	/** Log a raw message using the default resolved logger, if no logger is resolved in scope, the kernel log is used
	 * @param aLevel the level of the message
	 * @param aMessage the message text
	 */
	void logRaw(int aLevel, String aMessage);
	
	/** Log a raw message using a specific logger
	 * @param aLog path of resource of custom logger config (e.g. "/etc/system/CustomLogConfig.xml")
	 * @param aLevel the level of the message
	 * @param aMessage the message text
	 */
	void logRaw(String aLog, int aLevel, String aMessage);
	
	/** Log a formatted message
	 * @param aLevel the level of the message
	 * @param aMessageKey The id of the message that will be looked up
	 * @param aParams optional parameters to be substituted into the message
	 */
	void logFormatted(int aLevel, String aMessageKey, Object... aParams);
	
	/** Log a formatted message
	 * @param aLog path of resource of custom logger config (e.g. "/etc/system/CustomLogConfig.xml")
	 * @param aLevel the level of the message
	 * @param aMessageKey The id of the message that will be looked up
	 * @param aParams optional parameters to be substituted into the message
	 */
	void logFormatted(String aLog, int aLevel, String aMessageKey, Object... aParams);

}