/******************************************************************************
 * (c) Copyright 2002,2003, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.util;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import java.util.*;
import java.io.*;

/**
 * Thin wrapper over a DOM to provide fast XPath lookup on values in convenient ways
 * @author  tab
 */
public final class XMLReadable
{
	private Node mContextNode;
	
	private static DocumentBuilderFactory sFactory;
	private static DocumentBuilder sBuilder;
	
	static
	{	try
		{	sFactory = DocumentBuilderFactory.newInstance();
			sFactory.setNamespaceAware(true);
			sFactory.setValidating(false);
			sFactory.setAttribute("http://apache.org/xml/features/dom/defer-node-expansion", new Boolean(false));
		}
		catch(IllegalArgumentException iae) 	{;}
		
		try
		{	sBuilder = sFactory.newDocumentBuilder();
		} catch (ParserConfigurationException e)	{;	}
	}
	
	
	
	/** Creates a new instance of XMLReadable 
	  @param aContextNode the context node that relative expressions are evaluated against
	 */
	public XMLReadable(Node aContextNode)
	{	mContextNode = aContextNode;
	}

	public XMLReadable(InputStream aIS)	throws IOException, SAXException
	{	InputSource is = new InputSource(aIS);
		synchronized(sBuilder)
		{	mContextNode= sBuilder.parse(is);
		}
	}
	
	public Node getContextNode()
	{	return mContextNode;
	}
	
	/** return a list of nodes that match the given expression */
	public List<Node> getNodes(String aXPath)
	{	return FastXPath.eval(mContextNode, aXPath);
	}
	
	public Node getFirstNode(String aXPath)
	{	return FastXPath.getSingleNode(mContextNode, aXPath);
	}
	
	/** return a list of strings that are the text values of elements matching the given expression
	 * @param aTrim if true trims the resulting strings of whitespace
	 */
	public List<String> getTexts(String aXPath, boolean aTrim)
	{	List nodes = FastXPath.eval(mContextNode, aXPath);
		for (int i=0; i<nodes.size(); i++ )
		{	Node n = (Node)nodes.get(i);
			String text = getText(n);
			if (aTrim) text=text.trim();
			nodes.set(i,text);
		}
		return nodes;
	}
	
	/** return a string value of the text of a single node, returns an empty string if node not found */
	public String getText(String aXPath)
	{	String result="";
		Node n = FastXPath.getSingleNode(mContextNode, aXPath);
		if (n!=null)
		{	result = getText(n);
		}
		return result;
	}
	
	public String getTrimText(String aXPath)
	{	return getText(aXPath).trim();
	}
	
	/** return an integer from text value of the text at a node given by the expression.
	 * if the node isn't found or fails to parse as a number, the default value is used
	 */ 
	public int getInt(String aXPath, int aDefault)
	{	try
		{	Node n = FastXPath.getSingleNode(mContextNode, aXPath);
			String s = getText(n);
			return Integer.parseInt(s);
		}	catch (Exception e)
		{	//SysLogger.log(SysLogger.WARNING, this, aXPath+" eval failed on config");
			return aDefault;
		}
	}
	
	private static String getText(Node aNode)
	{   StringBuffer result = null;
		String result2 = null;
		for (Node n=aNode.getFirstChild(); n!=null; n=n.getNextSibling())
		{   if (n.getNodeType()==Node.TEXT_NODE || n.getNodeType()==Node.CDATA_SECTION_NODE)
			{   String s = n.getNodeValue();
				if (result2==null)
				{	result2 = s;
				}
				else if (result==null)
				{	result = new StringBuffer(result2.length()+s.length());
					result.append(result2);
					result.append(s);
				}
				else
				{	result.append(s);
				}
			}
		}
		if (result==null)
		{	if (result2==null)
			{	result2="";
			}
		}
		else
		{	result2 = new String(result);
		}
		return result2;
	}
}